<?php

namespace EOG\Models;

class Stripe
{
    const TEAM_BLUE = 'blue';
    const TEAM_ORANGE = 'orange';

    private $allowedColors = [self::TEAM_BLUE, self::TEAM_ORANGE];
    private $allowedStripeClasses = ['team', 'simple'];
    private $allowedOperators = [
        'alibi', 'amaru', 'ash', 'bandit', 'blackbeard', 'blitz', 'buck', 'capitao', 'castle',
        'caveira', 'clash', 'doc', 'dokkaebi', 'echo', 'ela', 'finka', 'frost', 'fuze',
        'glaz', 'goyo', 'gridlock', 'hibana', 'iq', 'jackal', 'jager', 'kaid', 'kapkan',
        'lesion', 'lion', 'maestro', 'maverick', 'mira', 'montagne', 'mozzie', 'mute', 'nokk',
        'nomad', 'pulse', 'rook', 'sledge', 'smoke', 'tachanka', 'thatcher', 'thermite', 'twitch',
        'valkyrie', 'vigil', 'warden', 'ying', 'zofia'];
    protected $state = [
        'stripe' => [
            'class' => 'team',
        ],
        'cup' => [
            'number' => '#75',
            'name' => '5on5 Open Cup',
        ],
        'team' => [
            'orange' =>
                [
                    'name' => 'Narancs Csapat',
                    'ban' => [
                        'mira',
                        'jackal'
                    ]
                ],
            'blue' => [
                'name' => 'Kék csapat',
                'ban' => [
                    'rook',
                    'blitz'
                ]
            ]
        ]
    ];

    public function setClass(string $class): self
    {
        if (!in_array($class, $this->allowedStripeClasses)) {
            throw new \InvalidArgumentException("Given class " . $class . " not allowed. Allowed classes: " . implode(', ', $this->allowedStripeClasses));
        }

        if ($class === 'simple') {
            $this->setSimpleStripe();
        }

        $this->state['stripe']['class'] = $class;

        return $this;
    }

    private function setSimpleStripe()
    {
        unset($this->state['team']);
    }

    public function setCup(string $number, string $name)
    {
        $this->state['cup']['number'] = $number;
        $this->state['cup']['name'] = $name;
    }

    /**
     * @param string $color Csapat szín validáció
     *
     * Dob egy Exception-t ha nem orange / blue
     */
    private function testColor(string $color)
    {
        if (!in_array($color, $this->allowedColors)) {
            throw new \InvalidArgumentException("Given colour " . $team_color . " not allowed. Allowed colours: " . implode(', ', $this->allowedColors));
        }
    }

    public function setTeamName(string $team_color = self::TEAM_BLUE, $name)
    {
        $this->testColor($team_color);
        $this->state['team'][$team_color]['name'] = $name;
    }

    public function addTeamBan(string $team_color = self::TEAM_BLUE, $operator)
    {
        $this->testColor($team_color);
        if (!in_array($operator, $this->allowedOperators)) {
            //FIXME: védő és támadó operátorok külön
            throw new \InvalidArgumentException("Given operator not allowed:  " . $operator . " Allowed operators: " . implode(', ', $this->allowedOperators));
        }
    }

    public function __get($name)
    {
        if ($this->__isset($name)) {
            return $this->state[$name];
        }

        return null;
    }

    public function __isset($name)
    {
        return isset($this->state[$name]);
    }

    public function loadJson(string $json)
    {
        $state = json_decode($json);

        if (!empty($state['stripe']["class"])) {
            $this->setClass($state['stripe']["class"]);
        }

        if (!empty($state['cup']['name']) && !empty($state['cup']['number'])) {
            $this->setCup($state['cup']['number'], $state['cup']['name']);
        }

    }
}